@@ -17,11 +17,11 @@
 #![allow(clippy::module_name_repetitions)]
 
 use super::{
-    base_expr, eq::AstEq, query, ArrayPattern, ArrayPredicatePattern, AssignPattern, BinExpr,
-    BinOpKind, Bytes, ClauseGroup, Comprehension, ComprehensionCase, Costly, DefaultCase, EmitExpr,
-    EventPath, Expr, Expression, Field, FnDecl, FnDoc, Helper, Ident, IfElse, ImutExpr,
-    ImutExprInt, Invocable, Invoke, InvokeAggr, InvokeAggrFn, List, Literal, LocalPath, Match,
-    Merge, MetadataPath, ModDoc, NodeMetas, Patch, PatchOperation, Path, Pattern, PredicateClause,
+    base_expr, query, ArrayPattern, ArrayPredicatePattern, AssignPattern, BinExpr, BinOpKind,
+    Bytes, ClauseGroup, Comprehension, ComprehensionCase, Costly, DefaultCase, EmitExpr, EventPath,
+    Expr, Expression, Field, FnDecl, FnDoc, Helper, Ident, IfElse, ImutExpr, ImutExprInt,
+    Invocable, Invoke, InvokeAggr, InvokeAggrFn, List, Literal, LocalPath, Match, Merge,
+    MetadataPath, ModDoc, NodeMetas, Patch, PatchOperation, Path, Pattern, PredicateClause,
     PredicatePattern, Record, RecordPattern, Recur, ReservedPath, Script, Segment, StatePath,
     StrLitElement, StringLit, TestExpr, TuplePattern, UnaryExpr, UnaryOpKind, Warning,
 };
@@ -604,28 +604,16 @@ impl<'script> Upable<'script> for ExprRaw<'script> {
                 let path = a.path.up(helper)?;
                 let mid = helper.add_meta(a.start, a.end);
                 match a.expr.up(helper)? {
-                    Expr::Imut(ImutExprInt::Merge(m)) => {
-                        if path.ast_eq(&m.target) {
-                            Expr::MergeInPlace(Box::new(*m))
-                        } else {
-                            Expr::Assign {
-                                mid,
-                                path,
-                                expr: Box::new(ImutExprInt::Merge(m).into()),
-                            }
-                        }
-                    }
-                    Expr::Imut(ImutExprInt::Patch(m)) => {
-                        if path.ast_eq(&m.target) {
-                            Expr::PatchInPlace(Box::new(*m))
-                        } else {
-                            Expr::Assign {
-                                mid,
-                                path,
-                                expr: Box::new(ImutExprInt::Patch(m).into()),
-                            }
-                        }
-                    }
+                    Expr::Imut(ImutExprInt::Merge(m)) => Expr::Assign {
+                        mid,
+                        path,
+                        expr: Box::new(ImutExprInt::Merge(m).into()),
+                    },
+                    Expr::Imut(ImutExprInt::Patch(m)) => Expr::Assign {
+                        mid,
+                        path,
+                        expr: Box::new(ImutExprInt::Patch(m).into()),
+                    },
                     expr => Expr::Assign {
                         mid,
                         path,
@@ -1017,7 +1005,11 @@ impl<'script> Upable<'script> for ImutExprRaw<'script> {
                                         if let Some(_f) = l.value.as_f64() {
                                             StrLitElement::Lit("42".into())
                                         } else {
-                                            StrLitElement::Lit(l.value.encode().into())
+                                            StrLitElement::Lit(
+                                                crate::utils::sorted_serialize(&l.value)
+                                                    .unwrap_or_default()
+                                                    .into(),
+                                            )
                                         }
                                     },
                                     |s| StrLitElement::Lit(s.to_string().into()),
