@@ -13,24 +13,23 @@
 // limitations under the License.
 
 use super::{
-    merge_values, patch_value, resolve, resolve_value, set_local_shadow, test_guard,
-    test_predicate_expr, Env, ExecOpts, LocalStack, NULL,
+    resolve, resolve_value, set_local_shadow, test_guard, test_predicate_expr, Env, ExecOpts,
+    LocalStack, NULL,
 };
 use crate::errors::{
     error_assign_array, error_assign_to_const, error_bad_key_err, error_invalid_assign_target,
-    error_need_obj, error_need_obj_err, error_no_clause_hit, Result,
+    error_need_obj_err, error_no_clause_hit, Result,
 };
 use crate::prelude::*;
 use crate::registry::RECUR_PTR;
 use crate::{
     ast::{
         BaseExpr, ClauseGroup, ClausePreCondition, Comprehension, DefaultCase, EmitExpr, EventPath,
-        Expr, IfElse, ImutExprInt, Match, Merge, Patch, Path, Segment,
+        Expr, IfElse, ImutExprInt, Match, Path, Segment,
     },
     errors::error_oops_err,
 };
 use crate::{stry, Value};
-use matches::matches;
 use std::mem;
 use std::{
     borrow::{Borrow, Cow},
@@ -142,7 +141,6 @@ where
                     }
                 }
                 ClauseGroup::SearchTree { tree, rest, .. } => {
-                    let target: &Value<'script> = unsafe { mem::transmute(target) };
                     if let Some((e, l)) = tree.get(&target) {
                         return Expr::execute_effectors(opts, env, event, state, meta, local, e, l);
                     }
@@ -163,7 +161,6 @@ where
                                 }
                             }
                             ClauseGroup::SearchTree { tree, rest, .. } => {
-                                let target: &Value<'script> = unsafe { mem::transmute(target) };
                                 if let Some((e, l)) = tree.get(&target) {
                                     return Expr::execute_effectors(
                                         opts, env, event, state, meta, local, e, l,
@@ -228,103 +225,6 @@ where
         }
     }
 
-    #[allow(mutable_transmutes, clippy::transmute_ptr_to_ptr)]
-    fn patch_in_place(
-        &'script self,
-        opts: ExecOpts,
-        env: &'run Env<'run, 'event, 'script>,
-        event: &'run Value<'event>,
-        state: &'run Value<'static>,
-        meta: &'run Value<'event>,
-        local: &'run LocalStack<'event>,
-        expr: &'script Patch,
-    ) -> Result<Cow<'run, Value<'event>>> {
-        // This function is called when we encounter code that consumes a value
-        // to patch it. So the following code:
-        // ```tremor
-        // let event = patch event of insert "key" => "value" end
-        // ```
-        // When executed on it's own would clone the event, add a key and
-        // overwrite original event.
-        //
-        // We optimise this as:
-        // ```
-        // patch_in_place event of insert "key" => "value" end
-        // ```
-        //
-        // This code is generated in impl Upable for ExprRaw where the following
-        // checks are performed:
-        //
-        // 1) the patch is on the RHS of an assignment
-        // 2) the path of the assigned value and the path of the patched
-        //    expression are identical.
-        //
-        // In turn this guarantees (at compile time):
-        //
-        // 1) The target (`expr`) is a path lookup
-        // 2) The target is not a known constant as otherwise the assignment
-        //    will complan
-        // 3) this leave the `expr` to be either a local, the event, the state,
-        //    metadata or a subkey thereof.
-        //
-        // And the following guarantees at run time:
-        //
-        // 1) the `expr` is an existing key of the mentioned categories,
-        //    otherwise `expr.target.run` will error.
-        // 2) `value` will never be owned (however the resolve function is
-        //    generic so it needs to return a Cow)
-
-        let value: Cow<'run, Value<'event>> =
-            stry!(expr.target.run(opts, env, event, state, meta, local));
-        debug_assert!(
-            !matches!(value, Cow::Owned(_)),
-            "We should never see a owned value here as patch_in_place is only ever called on existing data in event, state, meta or local"
-        );
-        let v: &Value<'event> = value.borrow();
-        let v: &mut Value<'event> = unsafe { mem::transmute(v) };
-        stry!(patch_value(
-            self, opts, env, event, state, meta, local, v, expr
-        ));
-        Ok(value)
-    }
-
-    #[allow(mutable_transmutes, clippy::transmute_ptr_to_ptr)]
-    fn merge_in_place(
-        &'script self,
-        opts: ExecOpts,
-        env: &'run Env<'run, 'event, 'script>,
-        event: &'run mut Value<'event>,
-        state: &'run mut Value<'static>,
-        meta: &'run mut Value<'event>,
-        local: &'run mut LocalStack<'event>,
-        expr: &'script Merge,
-    ) -> Result<Cow<'run, Value<'event>>> {
-        // Please see the soundness reasoning in `patch_in_place` for details
-        // those functions perform the same function just with slighty different
-        // operations.
-        let value_cow: Cow<'run, Value<'event>> =
-            stry!(expr.target.run(opts, env, event, state, meta, local));
-        debug_assert!(
-            !matches!(value_cow, Cow::Owned(_)),
-            "We should never see a owned value here as merge_in_place is only ever called on existing data in event, state, meta or local"
-        );
-
-        if value_cow.is_object() {
-            let value: &Value<'event> = value_cow.borrow();
-            let value: &mut Value<'event> = unsafe { mem::transmute(value) };
-            let replacement = stry!(expr.expr.run(opts, env, event, state, meta, local,));
-
-            if replacement.is_object() {
-                stry!(merge_values(self, &expr.expr, value, &replacement));
-                Ok(value_cow)
-            } else {
-                error_need_obj(self, &expr.expr, replacement.value_type(), &env.meta)
-            }
-        } else {
-            error_need_obj(self, &expr.target, value_cow.value_type(), &env.meta)
-        }
-    }
-
     // TODO: Quite some overlap with `ImutExprInt::comprehension`
     fn comprehension(
         &'script self,
@@ -408,6 +308,7 @@ where
         }
     }
 
+    // ALLOW: https://github.com/tremor-rs/tremor-runtime/issues/1033
     #[allow(mutable_transmutes, clippy::transmute_ptr_to_ptr)]
     fn assign_nested(
         &'script self,
@@ -424,7 +325,7 @@ where
          * This function is icky we got to do some trickery here.
          * Since it's dangerous and icky it deserves some explanation
          * What we do here is we borrow the target we want to set
-         * as immutable and mem::transmute it to mutable where needed.
+         * as immutable and turn it to mutable where needed.
          *
          * We do this since there is no way to tell rust that it's safe
          * to borrow immutable out of something that's mutable even if
@@ -454,12 +355,10 @@ where
                 stry!(local
                     .get(lpath.idx, self, lpath.mid(), &env.meta)
                     .and_then(|o| {
-                        o.as_ref()
-                            .map(|l| -> &mut Value<'event> { unsafe { mem::transmute(l) } })
-                            .ok_or_else(|| {
-                                let key = env.meta.name_dflt(lpath.mid).to_string();
-                                error_bad_key_err(self, lpath, &path, key, vec![], &env.meta)
-                            })
+                        o.as_ref().ok_or_else(|| {
+                            let key = env.meta.name_dflt(lpath.mid).to_string();
+                            error_bad_key_err(self, lpath, &path, key, vec![], &env.meta)
+                        })
                     }))
             }
             Path::Meta(_path) => meta,
@@ -478,6 +377,7 @@ where
                 Segment::Id { key, .. } => {
                     current = stry!(key
                         .lookup_or_insert_mut(
+                            // ALLOW: https://github.com/tremor-rs/tremor-runtime/issues/1033
                             unsafe { mem::transmute::<&Value, &mut Value>(current) },
                             || Value::object_with_capacity(halfbrown::VEC_LIMIT_UPPER),
                         )
@@ -490,6 +390,7 @@ where
                 }
                 Segment::Element { expr, .. } => {
                     let id = stry!(expr.eval_to_string(opts, env, event, state, meta, local));
+                    // ALLOW: https://github.com/tremor-rs/tremor-runtime/issues/1033
                     let v: &mut Value<'event> = unsafe { mem::transmute(current) };
                     let map = stry!(v.as_object_mut().ok_or_else(|| error_need_obj_err(
                         self,
@@ -511,6 +412,7 @@ where
             }
         }
         unsafe {
+            // ALLOW: https://github.com/tremor-rs/tremor-runtime/issues/1033
             *mem::transmute::<&Value<'event>, &mut Value<'event>>(current) = value;
         }
         if opts.result_needed {
@@ -520,7 +422,6 @@ where
         }
     }
 
-    #[allow(mutable_transmutes, clippy::transmute_ptr_to_ptr)]
     fn assign_direct(
         &'script self,
         _opts: ExecOpts,
@@ -530,7 +431,7 @@ where
         _meta: &'run mut Value<'event>,
         local: &'run mut LocalStack<'event>,
         path: &'script Path,
-        mut value: Value<'event>,
+        value: Value<'event>,
     ) -> Result<Cow<'run, Value<'event>>> {
         match path {
             Path::Const(p) => {
@@ -539,23 +440,23 @@ where
             Path::Reserved(p) => {
                 error_assign_to_const(self, env.meta.name_dflt(p.mid()).into(), &env.meta)
             }
-            Path::Local(lpath) => match stry!(local.get(lpath.idx, self, lpath.mid(), &env.meta)) {
-                Some(l) => {
-                    let l: &mut Value<'event> = unsafe { mem::transmute(l) };
-                    *l = value;
-                    Ok(Cow::Borrowed(l))
-                }
-                d => {
-                    let d: &mut Option<Value<'event>> = unsafe { mem::transmute(d) };
-                    *d = Some(value);
-                    if let Some(l) = d {
+            Path::Local(lpath) => {
+                match stry!(local.get_mut(lpath.idx, self, lpath.mid(), &env.meta)) {
+                    Some(l) => {
+                        *l = value;
                         Ok(Cow::Borrowed(l))
-                    } else {
-                        // ALLOW: we assign d in the line above the access, we know it is Some
-                        unreachable!()
+                    }
+                    d => {
+                        *d = Some(value);
+                        if let Some(l) = d {
+                            Ok(Cow::Borrowed(l))
+                        } else {
+                            // ALLOW: we assign d in the line above the access, we know it is Some
+                            unreachable!()
+                        }
                     }
                 }
-            },
+            }
             Path::Meta(_path) => error_invalid_assign_target(self, &env.meta),
             Path::Event(_path) => {
                 *event = value;
@@ -566,10 +467,7 @@ where
                 // object keys in value to be owned COW's). This ensures that the current
                 // value is kept as part of state across subsequent state assignments (if
                 // users choose to do so).
-                value = value.into_static();
-                // for the compiler, the type of value still has 'event as the lifetime
-                // so we transmute it to conform with the lifetime of state ('static).
-                *state = unsafe { mem::transmute(value) };
+                *state = value.into_static();
                 Ok(Cow::Borrowed(state))
             }
         }
@@ -657,12 +555,6 @@ where
             }
             Expr::Match(ref expr) => self.match_expr(opts, env, event, state, meta, local, expr),
             Expr::IfElse(ref expr) => self.if_expr(opts, env, event, state, meta, local, expr),
-            Expr::MergeInPlace(ref expr) => self
-                .merge_in_place(opts, env, event, state, meta, local, expr)
-                .map(Cont::Cont),
-            Expr::PatchInPlace(ref expr) => self
-                .patch_in_place(opts, env, event, state, meta, local, expr)
-                .map(Cont::Cont),
             Expr::Comprehension(ref expr) => {
                 self.comprehension(opts, env, event, state, meta, local, expr)
             }
